# Multilanguage Form Display

## History
The original module was called the [Dual Language Editor](https://www.uottawa.ca/uoweb/en/user-guide/exploring-dual-language-editor) (DLE). It was made for Drupal 7 and had a lot of work-arounds to try and solve a problem which was made nearly insurmountable by the proceedural coding.

With the introduction of Drupal 8 and the Object Oriented nature of the Symfony framework it leverages, we can now better leverage the various APIs and drastically reduce the lines of code from the previous version of the module.

## Description:

The purpose of this module is to extend the entity forms for a multilingual site, thereby allowing an administrator to select whether a content type or entity can display for the content editor all translatable fields in one form. The module will permit the leveraging of [Display Suite](https://www.drupal.org/project/ds) and [Panels](https://www.drupal.org/project/panels) if need be.

Due to the nature of the Field and Form API. We have to be careful not to trample all over the core and contrib widgets for fields. The best we can achieve is to play nicely with the majority of modules that enable and extend Drupal's fields.

### The new approach

We're using the FieldPlugin classes approach.

### The previous method

(**_n.b.: all this was several iterations of code ago_**)

To achieve this goal, we are focusing on altering a few forms and calling on the power of OOP to re-purpose some fields, move them around and eventually get them to save back to their proper storage definitions.

We do so by calling on each translation, getting the FormState to play nicely with each translation and get Drupal to sort itself out without using a heavy hammer approach at getting and setting values. We leave the power of the classes to manage this. Go Methods!!

To understand what is going on here, just follow the trail of breadcrumbs. We start at the hook_widget_form_alter(). Here we begin the process for each field which is permitted to be translated:

- From the translated entity we get the appropriate field as an item.
- We tell the FormState not to recurse as we will be asking the widget for a form
- create a basic form
- Tell the FormState to shift language state
- get the widget's form
- iterate through that until we have all the languages set and saved in the FormState

Now we can more onto the hook_node_form_alter() to get the NodeForm modifications done. These will ensure that we have what we need to Render a form with the proper named fields.

Since we told the widget we are being used by us (#multiform_display_use) we don't need to repeat some of the housework we did in the widget alter function. Now we can just start sweeping through the form object, cleaning it up for each language instance and getting the proper values to the stored in each translated entity.

The only caveat is we need to unset a few values and clean up a few #field_name and #parent properties to ensure no overlapping naming conventions. Once this is done, we're home free. The remaining functionality (validation and submission) is handled by their appropriate objects. We only have to ensure that on submit
we cycle through the language fields we've created and force their values into their none languaged equivalents in their appropriate translation entity->save().

## Module project page:
http://drupal.org/project/mfd (TBD)

## Documentation page (D8 version):
http://drupal.org/node/##


### Configuration

1. Its really quite simple, install, enable more than one language and activate the entity's translatable fields you wish to use this feature with.. 

## Module Creator

wilco - https://www.drupal.org/u/wilco